from datetime import datetime
from typing import Tuple

import pymongo as mongo
import polars as pl

from pymongoarrow.api import Schema, find_polars_all

from pulseel.utils.models import IncrementalConfig, ExtractionInfo


def get_mongo_data(mongo_uri: str,
                   database: str,
                   collection: str,
                   schema: Schema | None = None,
                   incremental: IncrementalConfig | None = None
                   ) -> Tuple[pl.DataFrame, ExtractionInfo]:

        client = mongo.MongoClient(mongo_uri)
        db = client[database]
        collection = db[collection]

        iop = {incremental.column_name: {"$gt": incremental.last_value}} if incremental else {}

        stime = datetime.now()
        # TODO Handle Large Collections with Partitioning/Sharding
        df = find_polars_all(collection, query=iop, schema=schema)
        etime = datetime.now()

        exec_time = int((etime - stime).total_seconds())

        last_value = df.select(pl.col(incremental.column_name).max())[0, 0] if incremental else None

        return df, ExtractionInfo(rows_extracted=df.shape[0], execution_in_seconds=exec_time, inc_last_value=last_value)


