import datetime
import os
import tempfile
import uuid
from typing import List

import polars as pl

from azure.storage.blob import BlobServiceClient
from pulseel.utils.models import ADLSConfig



def upload_to_blob_storage(data: pl.DataFrame | List[pl.DataFrame], path: str, config: ADLSConfig):

    _id = uuid.uuid4()

    with tempfile.TemporaryDirectory() as tmpdir:

        if path.endswith("/"):
            path = path[:-1]

        if not isinstance(data, list):
            data = [data]

        blob_client = BlobServiceClient.from_connection_string(config.connection_string)
        container_client = blob_client.get_container_client(config.container_name)

        # Marks the start of writing
        container_client.upload_blob(os.path.join(path, f"{_id}__started"), datetime.datetime.now().isoformat()),


        for i, df in enumerate(data):
            filename = f"{_id}-{i}.parquet"
            abs_path = os.path.join(tmpdir, filename)
            with open(os.path.join(tmpdir, filename), "wb") as f:
                df.write_parquet(f)

            with open(abs_path, "rb") as data:
                container_client.upload_blob(os.path.join(path, filename), data)

        # Marks that writing was completed
        container_client.upload_blob(os.path.join(path, f"{_id}__completed"), datetime.datetime.now().isoformat())


