import os
import tempfile
import uuid
from typing import List
import adlfs
from pulseel.extractors.rdbms.mssql import get_data, get_data_incremental
from pulseel.utils.helpers import get_sqlalchemy_uri
from pulseel.utils.models import DatabaseConfig, ExtractionInfo, IncrementalConfig
from sqlalchemy import create_engine



def mssql_to_az(db_conf: DatabaseConfig,
                query: str | List[str],
                az_path: str,
                az_options: dict = {},
                inc_conf: IncrementalConfig | None = None
                ):

    engine = create_engine(get_sqlalchemy_uri(db_conf))
    # fs = adlfs.AzureBlobFileSystem(**az_options)

    if isinstance(query, str):
        query = [query]

    info_list = []


    # Extract each query to a local parquet. After all queries concludes, copy all files to azure
    with tempfile.TemporaryDirectory() as tmpdir:
        print(tmpdir)
        # TODO - Make a retry for each query
        for i, q in enumerate(query):
            print(f"Executing query {i+1}: {q}")
            if inc_conf:
                data, info = get_data_incremental(engine, q, inc_conf.column_name, inc_conf.last_value)

            else:
                data, info = get_data(engine, q)

            info_list.append(info)

            # Normalize path
            if az_path.endswith("/"):
                az_path = az_path[:-1]
            filename = f"{uuid.uuid4()}-{i}.parquet"

            with open(os.path.join(tmpdir, filename) , "wb") as f:
                data.write_parquet(f)

        # Copy all files to azure blob storage
        fs = adlfs.AzureBlobFileSystem(**az_options)
        fs.put(tmpdir + "/*", az_path)



    # Compute aggregated metrics
    sum_exec_time = sum([e.execution_in_seconds for e in info_list])
    sum_rows = sum([e.rows_extracted for e in info_list])
    max_inc_value = str(max([int(e.inc_last_value) for e in info_list])) if inc_conf else None

    print(ExtractionInfo(rows_extracted=sum_rows, execution_in_seconds=sum_exec_time, inc_last_value=max_inc_value))

# TODO Build a argument parsing cli







